@@ -1,5 +1,5 @@
-# GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
+# GNU Solfege - ear training for GNOME
+# Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,20 +18,26 @@
 import gtk, gnome
 import gu
 import inputwidgets
+import widgets
 import soundcard, mpd
 import dataparser
 
 import const, utils, string
+import os
+import sys
+import signal
 import types
 import cfg
 
 class Teacher(cfg.ConfigUtils):
-    def __init__(self, exname, app):
+    def __init__(self, exname, app, config):
         cfg.ConfigUtils.__init__(self, exname)
         self.m_app = app
         self.q_status = const.QSTATUS_NO
         self.m_statistics = None
         self.m_timeout_handle = None
+        for k in config.keys():
+            self.set_string(k, dict[k])
     def get_instrument_config(self, num):
         if self.get_bool('override_default_instrument'):
             instr_low = self.get_int('lowest_instrument')
@@ -56,7 +62,7 @@ class Teacher(cfg.ConfigUtils):
                 def remove_timeout(self=self):
                     self.m_timeout_handle = None
                     self.g_view.new_question()
-                self.m_timeout_handle = gtk.timeout_add(int(self.get_float('seconds_before_new_question')*1000),  remove_timeout)
+                self.m_timeout_handle = gtk.timeout_add((self.get_int('seconds_before_new_question')*1000),  remove_timeout)
     def end_practise(self):
         if self.m_timeout_handle:
             gtk.timeout_remove(self.m_timeout_handle)
@@ -77,8 +83,8 @@ class MelodicIntervalTeacher(Teacher):
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
-    def __init__(self, exname, app):
-        Teacher.__init__(self, exname, app)
+    def __init__(self, exname, app, config):
+        Teacher.__init__(self, exname, app, config)
         self.m_tonika = None
         self.m_question = []
     def new_question(self, L, H):
@@ -155,22 +161,36 @@ class MelodicIntervalTeacher(Teacher):
         soundcard.synth.play_track(m)
 
 class LessonbasedTeacher(Teacher):
-    def __init__(self, exname, app, content_type):
-        Teacher.__init__(self, exname, app)
+    def __init__(self, exname, app, lesson_collection, lesson_file, config):
+        Teacher.__init__(self, exname, app, config)
         self.m_P = None
-        self.m_content_type = content_type
         self.m_question = None
+        if lesson_collection or lesson_file:
+            if lesson_collection:
+                self.set_string('lessoncollection', lesson_collection)
+            if lesson_file:
+                self.set_string('lessonfile', lesson_file)
         self.__make_sure_existing_lessonfile_is_selected()
-    def set_lessoncollection(self, collection):
-        self.set_string('lessoncollection', collection)
+    def set_lessonfile(self, lessoncollection, lessonfile):
+        """
+        Set the config variables 'lessonfile' and 'lessoncollection'
+        Parse the lesson file and save the statistics.
+        """
+        if lessonfile is not None \
+                and lessonfile == self.get_string('lessonfile'):
+            new_file = 0
+        else:
+            new_file = 1
+        if lessoncollection:
+            self.set_string('lessoncollection', lessoncollection)
+        if lessonfile:
+            self.set_string('lessonfile', lessonfile)
         self.__make_sure_existing_lessonfile_is_selected()
         self.parse_lessonfile()
-        if self.m_statistics:
-            self.m_statistics.lessonfile_changed(
-             self.get_string('lessoncollection'), self.get_string('lessonfile'))
-    def set_lessonfile(self, file):
-        self.set_string('lessonfile', file)
-        self.parse_lessonfile()
+        if new_file and self.m_P.header.questions_selectable:
+            self.block_watch('ask_for_names')
+            self.set_list('ask_for_names', range(0, len(self.m_P.get_unique_cnames())))
+            self.unblock_watch('ask_for_names')
         if self.m_statistics:
             self.m_statistics.lessonfile_changed(
              self.get_string('lessoncollection'), self.get_string('lessonfile'))
@@ -180,7 +200,7 @@ class LessonbasedTeacher(Teacher):
                  not in self.m_app.lessonfile_manager.get_collections():
             self.set_string('lessoncollection', 'solfege')
         F = self.m_app.lessonfile_manager.get_filenames(
-                 self.get_string('lessoncollection'), self.m_content_type)
+                 self.get_string('lessoncollection'), self.m_exname)
         if F:
             if self.get_string('lessonfile') not in F:
                 self.set_string('lessonfile', F[0])
@@ -196,20 +216,21 @@ class LessonbasedTeacher(Teacher):
              self.get_string('lessoncollection'),
              self.get_string('lessonfile'))
         try:
-            self.m_P = self.lessonfileclass(fn)
+            self.m_P = self.lessonfileclass(fn, self)
         except IOError:
             self.m_app.m_ui.display_error_message(
 """There was an IOError while trying to parse a lesson file. The cause was probably you deleting or renaming a lesson file. Please don't do this while Solfege is running. The program is still a little fragile. 
 
 You really should reastart the program now.""")
-    def play_question(self):
-        if self.q_status == const.QSTATUS_NO:
-            return
-        mpd.play_music(self.m_P.get_music(),
-                           self.m_P.get_tempo(),
-                           self.get_int('config/preferred_instrument'),
-                           self.get_int('config/preferred_instrument_velocity'))
+        if self.m_P.header.musicformat == "cmdline":
+            run = self.m_app.m_ui.dialog_yesno(_("The lessonfile contain potentially dangerous code because the lesson file set the variable \"musicformat\" to \"cmdline\". Run anyway?"))
+            if not run:
+                self.m_P = None
     def configure_exercise(self, lessoncollection, lessonfile, dict):
+    	"""This function should not be called by the constructors.
+	abstract.Teacher and abstract.LessonbasedTeacher does the same job,
+	but in correct order.
+	"""
         if lessoncollection or lessonfile:
             if lessoncollection:
                 self.set_string('lessoncollection', lessoncollection)
@@ -244,14 +265,14 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
 
         self.action_area = gtk.HBox()
         self.action_area.show()
-        vbox.pack_end(self.action_area, gtk.FALSE)
+        vbox.pack_end(self.action_area, False)
 
         self.config_box = gtk.VBox()
         self.config_box.set_border_width(gnome.ui.PAD)
         self.config_box.show()
         if no_notebook:
             self.pack_start(vbox)
-            self.pack_start(self.config_box, gtk.FALSE)
+            self.pack_start(self.config_box, False)
             self.g_notebook = None
         else:
             self.g_notebook = gtk.Notebook()
@@ -292,9 +313,9 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
             else:
                 self.g_statview.update()
     def get_pretty_name(self):
-        return string.replace(const.exercisedata[self.m_exname][0], '_', '')
+        return self.m_t.exercise_data['exercise_pretty_name']
     def _add_auto_new_question_gui(self, box):
-        hbox = gu.bHBox(box, gtk.FALSE)
+        hbox = gu.bHBox(box, False)
         hbox.set_spacing(gnome.ui.PAD_SMALL)
         adj = gtk.Adjustment(0, 0, 10, 0.1, 1, 1)
         spin = gu.nSpinButton(self.m_exname, 'seconds_before_new_question',
@@ -307,11 +328,11 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
             label.set_sensitive(button.get_active())
         b = gu.nCheckButton(self.m_exname, 'new_question_automatically',
                             _("_New question automatically."), callback=f)
-        hbox.pack_start(b, gtk.FALSE)
+        hbox.pack_start(b, False)
         label.set_sensitive(b.get_active())
-        hbox.pack_start(label, gtk.FALSE)
+        hbox.pack_start(label, False)
         spin.set_sensitive(b.get_active())
-        hbox.pack_start(spin, gtk.FALSE)
+        hbox.pack_start(spin, False)
 
 class IntervalGui(Gui):
     """
@@ -324,13 +345,13 @@ class IntervalGui(Gui):
 
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
-        self.practise_box.pack_start(self.g_flashbar, gtk.FALSE)
+        self.practise_box.pack_start(self.g_flashbar, False)
         self.practise_box.set_spacing(gnome.ui.PAD)
 
         self.g_new_interval = gu.bButton(self.action_area, _("_New interval"),
                                           self.new_question)
         self.g_repeat = gu.bButton(self.action_area, _("_Repeat"), self.repeat_question)
-        self.g_repeat.set_sensitive(gtk.FALSE)
+        self.g_repeat.set_sensitive(False)
         self.setup_key_bindings()
     def select_inputwidget(self):
         """
@@ -384,10 +405,132 @@ class LessonbasedGui(Gui):
     def __init__(self, teacher, window):
         Gui.__init__(self, teacher, window)
     def get_pretty_name(self):
-        s = string.replace(const.exercisedata[self.m_exname][0], '_', '')
+        s = self.m_t.exercise_data['exercise_pretty_name']
         if self.get_string('lessoncollection') == 'solfege':
             return "%s - %s" % (s, self.get_string('lessonfile'))
         else:
             return "%s - %s/%s" % (s,
              self.get_string('lessoncollection'), self.get_string('lessonfile'))
+    def add_random_transpose_gui(self):
+        hbox = gu.bHBox(self.config_box, False, False)
+        label = gtk.Label(_("Random transpose:"))
+        label.show()
+        hbox.pack_start(label, False)
+        hbox.set_spacing(6)
+        self.g_random_transpose = gtk.Entry()
+        self.g_random_transpose.set_editable(False)
+        self.g_random_transpose.show()
+        hbox.pack_start(self.g_random_transpose)
+
+        button = gtk.Button(_("Change ..."))
+        button.show()
+        button.connect('clicked', self.run_random_transpose_dialog)
+        hbox.pack_start(button)
+    def run_random_transpose_dialog(self, widget):
+        dlg = RandomTransposeDialog(self.m_t, self.g_win)
+        response = dlg.run()
+        if response == gtk.RESPONSE_OK:
+            self.m_t.m_P.header.random_transpose = dlg.get_value()
+            self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
+        dlg.destroy()
+    def add_select_lessonfile_gui(self):
+        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
+            self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
+        self.g_lessonfile_selector.show()
+        self.config_box.pack_start(self.g_lessonfile_selector)
+
+
+class RandomTransposeDialog(gtk.Dialog):
+    def __init__(self, teacher, parent):
+        gtk.Dialog.__init__(self, _("Set transposition"), parent, 0, 
+           (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+            gtk.STOCK_OK, gtk.RESPONSE_OK,
+            ))
+        self.m_t = teacher
+        dlg_vbox = gu.hig_dlg_vbox()
+        self.vbox.pack_start(dlg_vbox)
+        xbox, vbox = gu.hig_category_vbox(_("Select how to do random transposition"))
+        dlg_vbox.pack_start(xbox)
+        label = gtk.Label(_("""You can read about the different types of transposition in the lesson file documentation available on the Help menu."""))
+        label.set_line_wrap(True)
+        vbox.pack_start(label)
+        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        self.m_buttons = {}
+        self.m_spins = {}
+        self.m_buttons['no'] = b1 = gtk.RadioButton(None, _("No"))
+        self.m_spins['no'] = []
+        vbox.pack_start(b1)
+        self.m_buttons['yes'] = b2 = gtk.RadioButton(b1, _("Yes"))
+        self.m_spins['yes'] = []
+        vbox.pack_start(b2)
+
+        self.m_buttons['accidentals'] = gtk.RadioButton(b1, "accidentals")
+        hbox = gu.bHBox(vbox)
+        hbox.pack_start(self.m_buttons['accidentals'])
+        self.m_spins['accidentals'] = [
+            gtk.SpinButton(gtk.Adjustment(0, -7, 7, 1, 1, 1), 0.0, 0),
+            gtk.SpinButton(gtk.Adjustment(0, -7, 7, 1, 1, 1), 0.0, 0)]
+        hbox.pack_start(self.m_spins['accidentals'][0])
+        hbox.pack_start(self.m_spins['accidentals'][1])
+        gu.SpinButtonRangeController(self.m_spins['accidentals'][0],
+                                     self.m_spins['accidentals'][1],
+                                     -7, 7)
+
+        self.m_buttons['key'] = gtk.RadioButton(b1, "key")
+        hbox = gu.bHBox(vbox)
+        hbox.pack_start(self.m_buttons['key'])
+        self.m_spins['key'] = [
+            gtk.SpinButton(gtk.Adjustment(0, -10, 10, 1, 1, 1), 0.0, 0),
+            gtk.SpinButton(gtk.Adjustment(0, -10, 10, 1, 1, 1), 0.0, 0)]
+        hbox.pack_start(self.m_spins['key'][0])
+        hbox.pack_start(self.m_spins['key'][1])
+        gu.SpinButtonRangeController(self.m_spins['key'][0],
+                                     self.m_spins['key'][1],
+                                     -10, 10)
+
+        self.m_buttons['semitones'] = gtk.RadioButton(b1, "semitones")
+        hbox = gu.bHBox(vbox)
+        hbox.pack_start(self.m_buttons['semitones'])
+        self.m_spins['semitones'] = [
+            gtk.SpinButton(gtk.Adjustment(0, -100, 100, 1, 1, 1), 0.0, 0),
+            gtk.SpinButton(gtk.Adjustment(0, -100, 100, 1, 1, 1), 0.0, 0)]
+        hbox.pack_start(self.m_spins['semitones'][0])
+        hbox.pack_start(self.m_spins['semitones'][1])
+        gu.SpinButtonRangeController(self.m_spins['semitones'][0],
+                                     self.m_spins['semitones'][1],
+                                     -100, 100)
+        for n, w in self.m_buttons.items():
+            sizegroup.add_widget(w)
+            for ww in self.m_spins[n]:
+                ww.set_sensitive(False)
+            w.connect('clicked', self.on_spins_clicked, n)
+        if self.m_t.m_P.header.random_transpose == False:
+            k = 'no'
+        elif self.m_t.m_P.header.random_transpose == True:
+            k = 'yes'
+        else:
+            k = self.m_t.m_P.header.random_transpose[0]
+        self.m_buttons[k].set_active(True)
+        if k in ('accidentals', 'key', 'semitones'):
+            t, v1, v2 = self.m_t.m_P.header.random_transpose
+            # FIXME Because of the RangeController, we have to do this
+            # twice to be sure both values are set properly.
+            self.m_spins[t][1].set_value(v2)
+            self.m_spins[t][0].set_value(v1)
+            self.m_spins[t][1].set_value(v2)
+            self.m_spins[t][0].set_value(v1)
+        self.show_all()
+    def on_spins_clicked(self, w, n):
+        for w in self.m_spins[n]:
+            w.set_sensitive(self.m_buttons[n].get_active())
+    def get_value(self):
+        for n, btn in self.m_buttons.items():
+            if btn.get_active():
+                s = n
+        if s == 'yes':
+            return True
+        elif s == 'no':
+            return False
+        return [s, self.m_spins[s][0].get_value_as_int(), self.m_spins[s][1].get_value_as_int()]
+
 
